@@ -216,7 +216,8 @@ int create(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   }
 
   CLS_LOG(20, "create object_prefix=%s size=%llu order=%u features=%llu",
-	  object_prefix.c_str(), size, order, features);
+	  object_prefix.c_str(), (unsigned long long)size, order,
+	  (unsigned long long)features);
 
   if (features & ~RBD_FEATURES_ALL) {
     return -ENOSYS;
@@ -286,7 +287,7 @@ int get_features(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return -EINVAL;
   }
 
-  CLS_LOG(20, "get_features snap_id=%llu", snap_id);
+  CLS_LOG(20, "get_features snap_id=%llu", (unsigned long long)snap_id);
 
   if (snap_id == CEPH_NOSNAP) {
     int r = read_key(hctx, "features", &features);
@@ -328,7 +329,8 @@ int require_feature(cls_method_context_t hctx, uint64_t need)
   if (r < 0)
     return r;
   if ((features & need) != need) {
-    CLS_LOG(10, "require_feature missing feature %llx, have %llx", need, features);
+    CLS_LOG(10, "require_feature missing feature %llx, have %llx",
+            (unsigned long long)need, (unsigned long long)features);
     return -ENOEXEC;
   }
   return 0;
@@ -355,7 +357,7 @@ int get_size(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return -EINVAL;
   }
 
-  CLS_LOG(20, "get_size snap_id=%llu", snap_id);
+  CLS_LOG(20, "get_size snap_id=%llu", (unsigned long long)snap_id);
 
   int r = read_key(hctx, "order", &order);
   if (r < 0) {
@@ -413,7 +415,8 @@ int set_size(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return r;
   }
 
-  CLS_LOG(20, "set_size size=%llu orig_size=%llu", size);
+  CLS_LOG(20, "set_size size=%llu orig_size=%llu", (unsigned long long)size,
+          (unsigned long long)orig_size);
 
   bufferlist sizebl;
   ::encode(size, sizebl);
@@ -489,7 +492,8 @@ int get_protection_status(cls_method_context_t hctx, bufferlist *in,
   if (r < 0)
     return r;
 
-  CLS_LOG(20, "get_protection_status snap_id=%llu", snap_id.val);
+  CLS_LOG(20, "get_protection_status snap_id=%llu",
+         (unsigned long long)snap_id.val);
 
   if (snap_id == CEPH_NOSNAP)
     return -EINVAL;
@@ -505,7 +509,7 @@ int get_protection_status(cls_method_context_t hctx, bufferlist *in,
 
   if (snap.protection_status >= RBD_PROTECTION_STATUS_LAST) {
     CLS_ERR("invalid protection status for snap id %llu: %u",
-	    snap_id.val, snap.protection_status);
+	    (unsigned long long)snap_id.val, snap.protection_status);
     return -EIO;
   }
 
@@ -550,14 +554,14 @@ int set_protection_status(cls_method_context_t hctx, bufferlist *in,
   }
 
   CLS_LOG(20, "set_protection_status snapid=%llu status=%u",
-	  snap_id.val, status);
+	  (unsigned long long)snap_id.val, status);
 
   if (snap_id == CEPH_NOSNAP)
     return -EINVAL;
 
   if (status >= RBD_PROTECTION_STATUS_LAST) {
     CLS_LOG(10, "invalid protection status for snap id %llu: %u",
-	    snap_id.val, status);
+	    (unsigned long long)snap_id.val, status);
     return -EINVAL;
   }
 
@@ -675,7 +679,7 @@ int set_stripe_unit_count(cls_method_context_t hctx, bufferlist *in, bufferlist
     CLS_ERR("failed to read the order off of disk: %s", strerror(r));
     return r;
   }
-  if ((1ull << order) % stripe_unit || stripe_unit > (1ull << order)) {
+  if ((1ull << order) % stripe_unit) {
     CLS_ERR("stripe unit %llu is not a factor of the object size %llu",
             (unsigned long long)stripe_unit, 1ull << order);
     return -EINVAL;
@@ -729,7 +733,7 @@ int get_parent(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   if (r < 0)
     return r;
 
-  CLS_LOG(20, "get_parent snap_id=%llu", snap_id);
+  CLS_LOG(20, "get_parent snap_id=%llu", (unsigned long long)snap_id);
 
   cls_rbd_parent parent;
   r = require_feature(hctx, RBD_FEATURE_LAYERING);
@@ -797,8 +801,9 @@ int set_parent(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return r;
   }
 
-  CLS_LOG(20, "set_parent pool=%lld id=%s snapid=%llu size=%llu",
-	  pool, id.c_str(), snapid.val, size);
+  CLS_LOG(20, "set_parent pool=%llu id=%s snapid=%llu size=%llu",
+	  (unsigned long long)pool, id.c_str(), (unsigned long long)snapid.val,
+	  (unsigned long long)size);
 
   if (pool < 0 || id.length() == 0 || snapid == CEPH_NOSNAP || size == 0) {
     return -EINVAL;
@@ -808,9 +813,10 @@ int set_parent(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
   cls_rbd_parent parent;
   r = read_key(hctx, "parent", &parent);
   if (r == 0) {
-    CLS_LOG(20, "set_parent existing parent pool=%lld id=%s snapid=%llu overlap=%llu",
-	    parent.pool, parent.id.c_str(), parent.snapid.val,
-	    parent.overlap);
+    CLS_LOG(20, "set_parent existing parent pool=%llu id=%s snapid=%llu"
+            "overlap=%llu", (unsigned long long)parent.pool, parent.id.c_str(),
+	    (unsigned long long)parent.snapid.val,
+	    (unsigned long long)parent.overlap);
     return -EEXIST;
   }
 
@@ -1110,7 +1116,7 @@ int get_snapcontext(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
       snapid_t snap_id = snap_id_from_key(*it);
       snap_ids.push_back(snap_id);
     }
-    if (keys.size() > 0)
+    if (!keys.empty())
       last_read = *(keys.rbegin());
   } while (r == max_read);
 
@@ -1163,7 +1169,7 @@ int get_snapshot_name(cls_method_context_t hctx, bufferlist *in, bufferlist *out
     return -EINVAL;
   }
 
-  CLS_LOG(20, "get_snapshot_name snap_id=%llu", snap_id);
+  CLS_LOG(20, "get_snapshot_name snap_id=%llu", (unsigned long long)snap_id);
 
   if (snap_id == CEPH_NOSNAP)
     return -EINVAL;
@@ -1205,7 +1211,8 @@ int snapshot_add(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return -EINVAL;
   }
 
-  CLS_LOG(20, "snapshot_add name=%s id=%llu", snap_meta.name.c_str(), snap_meta.id.val);
+  CLS_LOG(20, "snapshot_add name=%s id=%llu", snap_meta.name.c_str(),
+	 (unsigned long long)snap_meta.id.val);
 
   if (snap_meta.id > CEPH_MAXSNAP)
     return -EINVAL;
@@ -1250,18 +1257,19 @@ int snapshot_add(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
 	::decode(old_meta, iter);
       } catch (const buffer::error &err) {
 	snapid_t snap_id = snap_id_from_key(it->first);
-	CLS_ERR("error decoding snapshot metadata for snap_id: %llu", snap_id.val);
+	CLS_ERR("error decoding snapshot metadata for snap_id: %llu",
+	        (unsigned long long)snap_id.val);
 	return -EIO;
       }
       if (snap_meta.name == old_meta.name || snap_meta.id == old_meta.id) {
 	CLS_LOG(20, "snap_name %s or snap_id %llu matches existing snap %s %llu",
-		snap_meta.name.c_str(), snap_meta.id.val,
-		old_meta.name.c_str(), old_meta.id.val);
+		snap_meta.name.c_str(), (unsigned long long)snap_meta.id.val,
+		old_meta.name.c_str(), (unsigned long long)old_meta.id.val);
 	return -EEXIST;
       }
     }
 
-    if (vals.size() > 0)
+    if (!vals.empty())
       last_read = vals.rbegin()->first;
   } while (r == RBD_MAX_KEYS_READ);
 
@@ -1312,7 +1320,7 @@ int snapshot_remove(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return -EINVAL;
   }
 
-  CLS_LOG(20, "snapshot_remove id=%llu", snap_id.val);
+  CLS_LOG(20, "snapshot_remove id=%llu", (unsigned long long)snap_id.val);
 
   // check if the key exists. we can't rely on remove_key doing this for
   // us, since OMAPRMKEYS returns success if the key is not there.
@@ -1709,7 +1717,7 @@ int dir_list(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
       if (images.size() >= max_return)
 	break;
     }
-    if (vals.size() > 0) {
+    if (!vals.empty()) {
       last_read = dir_key_for_name(images.rbegin()->first);
     }
   }
@@ -1990,7 +1998,8 @@ int rbd_assign_bid(cls_method_context_t hctx, bufferlist *in, bufferlist *out)
     return rc;
 
   if (rc && rc < (int)sizeof(info)) {
-    CLS_ERR("bad rbd_info object, read %d bytes, expected %d", rc, sizeof(info));
+    CLS_ERR("bad rbd_info object, read %d bytes, expected %d", rc,
+	    (int)sizeof(info));
     return -EIO;
   }
 
