@@ -29,30 +29,6 @@
 // -----------------------
 namespace ceph {
 
-/*
- * FormatterAttrs(const char *attr, ...)
- *
- * Requires a list of of attrs followed by NULL. The attrs should be char *
- * pairs, first one is the name, second one is the value. E.g.,
- *
- * FormatterAttrs("name1", "value1", "name2", "value2", NULL);
- */
-FormatterAttrs::FormatterAttrs(const char *attr, ...)
-{
-  const char *s = attr;
-  va_list ap;
-  va_start(ap, attr);
-  do {
-    const char *val = va_arg(ap, char *);
-    if (!val)
-      break;
-
-    attrs.push_back(make_pair(std::string(s), std::string(val)));
-    s = va_arg(ap, char *);
-  } while (s);
-  va_end(ap);
-}
-
 Formatter::Formatter()
 {
 }
@@ -275,32 +251,22 @@ void XMLFormatter::reset()
 
 void XMLFormatter::open_object_section(const char *name)
 {
-  open_section_in_ns(name, NULL, NULL);
-}
-
-void XMLFormatter::open_object_section_with_attrs(const char *name, const FormatterAttrs& attrs)
-{
-  open_section_in_ns(name, NULL, &attrs);
+  open_section_in_ns(name, NULL);
 }
 
 void XMLFormatter::open_object_section_in_ns(const char *name, const char *ns)
 {
-  open_section_in_ns(name, ns, NULL);
+  open_section_in_ns(name, ns);
 }
 
 void XMLFormatter::open_array_section(const char *name)
 {
-  open_section_in_ns(name, NULL, NULL);
-}
-
-void XMLFormatter::open_array_section_with_attrs(const char *name, const FormatterAttrs& attrs)
-{
-  open_section_in_ns(name, NULL, &attrs);
+  open_section_in_ns(name, NULL);
 }
 
 void XMLFormatter::open_array_section_in_ns(const char *name, const char *ns)
 {
-  open_section_in_ns(name, ns, NULL);
+  open_section_in_ns(name, ns);
 }
 
 void XMLFormatter::close_section()
@@ -308,15 +274,18 @@ void XMLFormatter::close_section()
   assert(!m_sections.empty());
   finish_pending_string();
 
-  print_spaces(false);
-  m_ss << "</" << m_sections.back() << ">";
+  std::string section = m_sections.back();
   m_sections.pop_back();
+  print_spaces();
+  m_ss << "</" << section << ">";
+  if (m_pretty)
+    m_ss << "\n";
 }
 
 void XMLFormatter::dump_unsigned(const char *name, uint64_t u)
 {
   std::string e(name);
-  print_spaces(true);
+  print_spaces();
   m_ss << "<" << e << ">" << u << "</" << e << ">";
   if (m_pretty)
     m_ss << "\n";
@@ -325,7 +294,7 @@ void XMLFormatter::dump_unsigned(const char *name, uint64_t u)
 void XMLFormatter::dump_int(const char *name, int64_t u)
 {
   std::string e(name);
-  print_spaces(true);
+  print_spaces();
   m_ss << "<" << e << ">" << u << "</" << e << ">";
   if (m_pretty)
     m_ss << "\n";
@@ -334,7 +303,7 @@ void XMLFormatter::dump_int(const char *name, int64_t u)
 void XMLFormatter::dump_float(const char *name, double d)
 {
   std::string e(name);
-  print_spaces(true);
+  print_spaces();
   m_ss << "<" << e << ">" << d << "</" << e << ">";
   if (m_pretty)
     m_ss << "\n";
@@ -343,26 +312,16 @@ void XMLFormatter::dump_float(const char *name, double d)
 void XMLFormatter::dump_string(const char *name, std::string s)
 {
   std::string e(name);
-  print_spaces(true);
+  print_spaces();
   m_ss << "<" << e << ">" << escape_xml_str(s.c_str()) << "</" << e << ">";
   if (m_pretty)
     m_ss << "\n";
 }
 
-void XMLFormatter::dump_string_with_attrs(const char *name, std::string s, const FormatterAttrs& attrs)
-{
-  std::string e(name);
-  std::string attrs_str;
-  get_attrs_str(&attrs, attrs_str);
-  print_spaces(true);
-  m_ss << "<" << e << attrs_str << ">" << escape_xml_str(s.c_str()) << "</" << e << ">";
-  if (m_pretty)
-    m_ss << "\n";
-}
-
 std::ostream& XMLFormatter::dump_stream(const char *name)
 {
   assert(m_pending_string_name.empty());
+  print_spaces();
   m_pending_string_name = name;
   m_ss << "<" << m_pending_string_name << ">";
   return m_pending_string;
@@ -377,7 +336,7 @@ void XMLFormatter::dump_format(const char *name, const char *fmt, ...)
   va_end(ap);
 
   std::string e(name);
-  print_spaces(true);
+  print_spaces();
   m_ss << "<" << e << ">" << escape_xml_str(buf) << "</" << e << ">";
   if (m_pretty)
     m_ss << "\n";
@@ -393,34 +352,17 @@ void XMLFormatter::write_raw_data(const char *data)
   m_ss << data;
 }
 
-void XMLFormatter::get_attrs_str(const FormatterAttrs *attrs, std::string& attrs_str)
+void XMLFormatter::open_section_in_ns(const char *name, const char *ns)
 {
-  std::stringstream attrs_ss;
-
-  for (std::list<std::pair<std::string, std::string> >::const_iterator iter = attrs->attrs.begin();
-       iter != attrs->attrs.end(); ++iter) {
-    std::pair<std::string, std::string> p = *iter;
-    attrs_ss << " " << p.first << "=" << "\"" << p.second << "\"";
-  }
-
-  attrs_str = attrs_ss.str();
-}
-
-void XMLFormatter::open_section_in_ns(const char *name, const char *ns, const FormatterAttrs *attrs)
-{
-  print_spaces(false);
-  std::string attrs_str;
-
-  if (attrs) {
-    get_attrs_str(attrs, attrs_str);
-  }
-
+  print_spaces();
   if (ns) {
-    m_ss << "<" << name << attrs_str << " xmlns=\"" << ns << "\">";
+    m_ss << "<" << name << " xmlns=\"" << ns << "\">";
   }
   else {
-    m_ss << "<" << name << attrs_str << ">";
+    m_ss << "<" << name << ">";
   }
+  if (m_pretty)
+    m_ss << "\n";
   m_sections.push_back(name);
 }
 
@@ -437,15 +379,12 @@ void XMLFormatter::finish_pending_string()
   }
 }
 
-void XMLFormatter::print_spaces(bool extra_space)
+void XMLFormatter::print_spaces()
 {
   finish_pending_string();
   if (m_pretty) {
-    std::vector<char> spaces(m_sections.size(), ' ');
-    if (extra_space)
-      spaces.push_back(' ');
-    spaces.push_back('\0');
-    m_ss << &spaces[0];
+    std::string spaces(m_sections.size(), ' ');
+    m_ss << spaces;
   }
 }
 
