@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include <sstream>
+#include <vector>
 
 
 const std::string BENCH_LASTRUN_METADATA = "benchmark_last_metadata";
@@ -113,10 +114,13 @@ void *ObjBencher::status_printer(void *_bencher) {
 	   << setw(10) << "last lat"
 	   << setw(10) << "avg lat" << std::endl;
     }
-    bandwidth = (double)(data.finished - previous_writes)
-      * (data.trans_size)
-      / (1024*1024)
-      / cycleSinceChange;
+    if (cycleSinceChange)
+      bandwidth = (double)(data.finished - previous_writes)
+	* (data.trans_size)
+	/ (1024*1024)
+	/ cycleSinceChange;
+    else
+      bandwidth = 0;
 
     if (!isnan(bandwidth)) {
       if (bandwidth > data.idata.max_bandwidth)
@@ -297,15 +301,16 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
   out(cout) << "Maintaining " << concurrentios << " concurrent writes of "
        << data.object_size << " bytes for at least "
        << secondsToRun << " seconds." << std::endl;
+  bufferlist* newContents = 0;
 
   std::string prefix = generate_object_prefix();
   out(cout) << "Object prefix: " << prefix << std::endl;
 
-  std::string name[concurrentios];
+  std::vector<string> name(concurrentios);
   std::string newName;
   bufferlist* contents[concurrentios];
   double total_latency = 0;
-  utime_t start_times[concurrentios];
+  std::vector<utime_t> start_times(concurrentios);
   utime_t stopTime;
   int r = 0;
   bufferlist b_write;
@@ -346,7 +351,6 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
 
   //keep on adding new writes as old ones complete until we've passed minimum time
   int slot;
-  bufferlist* newContents;
 
   //don't need locking for reads because other thread doesn't write
 
@@ -414,6 +418,7 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
     delete contents[slot];
     name[slot] = newName;
     contents[slot] = newContents;
+    newContents = 0;
   }
 
   while (data.finished < data.started) {
@@ -483,20 +488,19 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
   data.done = 1;
   lock.Unlock();
   pthread_join(print_thread, NULL);
+  delete newContents;
   return -5;
 }
 
 int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurrentios, int pid) {
-  data.finished = 0;
-
   lock_cond lc(&lock);
-  std::string name[concurrentios];
+  std::vector<string> name(concurrentios);
   std::string newName;
   bufferlist* contents[concurrentios];
   int index[concurrentios];
   int errors = 0;
   utime_t start_time;
-  utime_t start_times[concurrentios];
+  std::vector<utime_t> start_times(concurrentios);
   utime_t time_to_run;
   time_to_run.set_from_double(seconds_to_run);
   double total_latency = 0;
@@ -515,12 +519,14 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
     contents[i] = new bufferlist();
   }
 
+  lock.Lock();
+  data.finished = 0;
+  data.start_time = ceph_clock_now(g_ceph_context);
+  lock.Unlock();
+
   pthread_t print_thread;
   pthread_create(&print_thread, NULL, status_printer, (void *)this);
 
-  lock.Lock();
-  data.start_time = ceph_clock_now(g_ceph_context);
-  lock.Unlock();
   utime_t finish_time = data.start_time + time_to_run;
   //start initial reads
   for (int i = 0; i < concurrentios; ++i) {
@@ -700,7 +706,7 @@ int ObjBencher::clean_up(const std::string& prefix, int concurrentios) {
 
 int ObjBencher::clean_up(int num_objects, int prevPid, int concurrentios) {
   lock_cond lc(&lock);
-  std::string name[concurrentios];
+  std::vector<string> name(concurrentios);
   std::string newName;
   int r = 0;
   utime_t runtime;
@@ -840,7 +846,7 @@ bool ObjBencher::more_objects_matching_prefix(const std::string& prefix, std::li
 
   objects->clear();
 
-  while (objects->size() == 0) {
+  while (objects->empty()) {
     bool objects_remain = get_objects(&unfiltered_objects, 20);
     if (!objects_remain)
       return false;
@@ -860,7 +866,7 @@ bool ObjBencher::more_objects_matching_prefix(const std::string& prefix, std::li
 
 int ObjBencher::clean_up_slow(const std::string& prefix, int concurrentios) {
   lock_cond lc(&lock);
-  std::string name[concurrentios];
+  std::vector<string> name(concurrentios);
   std::string newName;
   int r = 0;
   utime_t runtime;
@@ -883,7 +889,7 @@ int ObjBencher::clean_up_slow(const std::string& prefix, int concurrentios) {
 
   //set up initial removes
   for (int i = 0; i < concurrentios; ++i) {
-    if (objects.size() == 0) {
+    if (objects.empty()) {
       // if there are fewer objects than concurrent ios, don't generate extras
       bool objects_found = more_objects_matching_prefix(prefix, &objects);
       if (!objects_found) {
@@ -935,7 +941,7 @@ int ObjBencher::clean_up_slow(const std::string& prefix, int concurrentios) {
     lock.Unlock();
 
     // get more objects if necessary
-    if (objects.size() == 0) {
+    if (objects.empty()) {
       objects_remain = more_objects_matching_prefix(prefix, &objects);
       // quit if there are no more
       if (!objects_remain) {
