@@ -1,7 +1,6 @@
 
 #include "CrushTester.h"
 
-// a better RNG to be found within
 #include <stdlib.h>
 
 
@@ -145,15 +144,16 @@ void CrushTester::adjust_weights(vector<__u32>& weight)
   }
 }
 
-bool CrushTester::check_valid_placement(int ruleno, vector<int> out, const vector<__u32>& weight){
+bool CrushTester::check_valid_placement(int ruleno, vector<int> in, const vector<__u32>& weight)
+{
 
   bool valid_placement = true;
   vector<int> included_devices;
   map<string,string> seen_devices;
 
   // first do the easy check that all devices are "up"
-  for (vector<int>::iterator it = out.begin(); it != out.end(); it++){
-    if (weight[(*it)] == 0){
+  for (vector<int>::iterator it = in.begin(); it != in.end(); it++) {
+    if (weight[(*it)] == 0) {
       valid_placement = false;
       break;
     } else if (weight[(*it)] > 0) {
@@ -173,8 +173,8 @@ bool CrushTester::check_valid_placement(int ruleno, vector<int> out, const vecto
 
   // get the smallest type id, and name
   int min_map_type = crush.get_num_type_names();
-  for (map<int,string>::iterator it = crush.type_map.begin(); it != crush.type_map.end(); it++ ){
-    if ( (*it).first < min_map_type ){
+  for (map<int,string>::iterator it = crush.type_map.begin(); it != crush.type_map.end(); it++ ) {
+    if ( (*it).first < min_map_type ) {
       min_map_type = (*it).first;
     }
   }
@@ -182,37 +182,37 @@ bool CrushTester::check_valid_placement(int ruleno, vector<int> out, const vecto
   string min_map_type_name = crush.type_map[min_map_type];
 
   // get the types of devices affected by RULENO
-  for (int i = 0; i < rule_size; i++){
+  for (int i = 0; i < rule_size; i++) {
     // get what operation is done by the current step
     int rule_operation = crush.get_rule_op(ruleno, i);
 
     // if the operation specifies choosing a device type, store it
-    if (rule_operation >= 2 && rule_operation != 4){
+    if (rule_operation >= 2 && rule_operation != 4) {
       int affected_type = crush.get_rule_arg2(ruleno,i);
       affected_types.push_back( crush.get_type_name(affected_type));
     }
   }
 
-  // find out if we are only dealing with osd's
-  bool only_osd_affected;
-  if (affected_types.size() == 1){
-    if ( (affected_types.back() == min_map_type_name) && (min_map_type_name == "osd") ){
+  // find in if we are only dealing with osd's
+  bool only_osd_affected = false;
+  if (affected_types.size() == 1) {
+    if ((affected_types.back() == min_map_type_name) && (min_map_type_name == "osd")) {
       only_osd_affected = true;
     }
   }
 
   // check that we don't have any duplicate id's
-  for (vector<int>::iterator it = included_devices.begin(); it != included_devices.end(); it++){
+  for (vector<int>::iterator it = included_devices.begin(); it != included_devices.end(); it++) {
     int num_copies = count(included_devices.begin(), included_devices.end(), (*it) );
-    if (num_copies > 1){
+    if (num_copies > 1) {
       valid_placement = false;
     }
   }
 
   // if we have more than just osd's affected we need to do a lot more work
-  if (!only_osd_affected){
+  if (!only_osd_affected) {
     // loop through the devices that are "in/up"
-    for (vector<int>::iterator it = included_devices.begin(); it != included_devices.end(); it++){
+    for (vector<int>::iterator it = included_devices.begin(); it != included_devices.end(); it++) {
       if (valid_placement == false)
         break;
 
@@ -220,13 +220,13 @@ bool CrushTester::check_valid_placement(int ruleno, vector<int> out, const vecto
       map<string,string> device_location_hierarchy = crush.get_full_location(*it);
 
       // loop over the types affected by RULENO looking for duplicate bucket assignments
-      for (vector<string>::iterator t = affected_types.begin(); t != affected_types.end(); t++){
-        if (seen_devices.count( device_location_hierarchy[*t] ) ){
+      for (vector<string>::iterator t = affected_types.begin(); t != affected_types.end(); t++) {
+        if (seen_devices.count( device_location_hierarchy[*t])) {
           valid_placement = false;
           break;
         } else {
           // store the devices we have seen in the form of (device name, device type)
-          seen_devices[ device_location_hierarchy[*t] ] = *t ;
+          seen_devices[ device_location_hierarchy[*t] ] = *t;
         }
       }
     }
@@ -242,10 +242,15 @@ int CrushTester::random_placement(int ruleno, vector<int>& out, int maxout, vect
   for (unsigned i = 0; i < weight.size(); i++)
     total_weight += weight[i];
 
+  if (total_weight == 0 ||
+      crush.get_max_devices() == 0)
+    return -EINVAL;
+
   // compute each device's proportional weight
   vector<float> proportional_weights( weight.size() );
-  for (unsigned i = 0; i < weight.size(); i++)
+  for (unsigned i = 0; i < weight.size(); i++) {
     proportional_weights[i] = (float) weight[i] / (float) total_weight;
+  }
 
   // determine the real maximum number of devices to return
   int devices_requested = min(maxout, get_maximum_affected_by_rule(ruleno));
@@ -277,8 +282,72 @@ int CrushTester::random_placement(int ruleno, vector<int>& out, int maxout, vect
   return 0;
 }
 
+void CrushTester::write_integer_indexed_vector_data_string(vector<string> &dst, int index, vector<int> vector_data)
+{
+  stringstream data_buffer (stringstream::in | stringstream::out);
+  unsigned input_size = vector_data.size();
 
+  // pass the indexing variable to the data buffer
+  data_buffer << index;
 
+  // pass the rest of the input data to the buffer
+  for (unsigned i = 0; i < input_size; i++) {
+    data_buffer << ',' << vector_data[i];
+  }
+
+  data_buffer << std::endl;
+
+  // write the data buffer to the destination
+  dst.push_back( data_buffer.str() );
+}
+
+void CrushTester::write_integer_indexed_vector_data_string(vector<string> &dst, int index, vector<float> vector_data)
+{
+  stringstream data_buffer (stringstream::in | stringstream::out);
+  unsigned input_size = vector_data.size();
+
+  // pass the indexing variable to the data buffer
+  data_buffer << index;
+
+  // pass the rest of the input data to the buffer
+  for (unsigned i = 0; i < input_size; i++) {
+    data_buffer << ',' << vector_data[i];
+  }
+
+  data_buffer << std::endl;
+
+  // write the data buffer to the destination
+  dst.push_back( data_buffer.str() );
+}
+
+void CrushTester::write_integer_indexed_scalar_data_string(vector<string> &dst, int index, int scalar_data)
+{
+  stringstream data_buffer (stringstream::in | stringstream::out);
+
+  // pass the indexing variable to the data buffer
+  data_buffer << index;
+
+  // pass the input data to the buffer
+  data_buffer << ',' << scalar_data;
+  data_buffer << std::endl;
+
+  // write the data buffer to the destination
+  dst.push_back( data_buffer.str() );
+}
+void CrushTester::write_integer_indexed_scalar_data_string(vector<string> &dst, int index, float scalar_data)
+{
+  stringstream data_buffer (stringstream::in | stringstream::out);
+
+  // pass the indexing variable to the data buffer
+  data_buffer << index;
+
+  // pass the input data to the buffer
+  data_buffer << ',' << scalar_data;
+  data_buffer << std::endl;
+
+  // write the data buffer to the destination
+  dst.push_back( data_buffer.str() );
+}
 
 int CrushTester::test()
 {
@@ -293,6 +362,11 @@ int CrushTester::test()
 
   // initial osd weights
   vector<__u32> weight;
+
+  /*
+   * note device weight is set by crushtool
+   * (likely due to a given a command line option)
+   */
   for (int o = 0; o < crush.get_max_devices(); o++) {
     if (device_weight.count(o)) {
       weight.push_back(device_weight[o]);
@@ -302,6 +376,7 @@ int CrushTester::test()
       weight.push_back(0);
     }
   }
+
   if (output_utilization_all)
     err << "devices weights (hex): " << hex << weight << dec << std::endl;
 
@@ -342,6 +417,11 @@ int CrushTester::test()
       int num_objects = ((max_x - min_x) + 1);
       float num_devices = (float) per.size(); // get the total number of devices, better to cast as a float here 
 
+      // create a structure to hold data for post-processing
+      tester_data_set tester_data;
+      vector<int> vector_data_buffer;
+      vector<float> vector_data_buffer_f;
+
       // create a map to hold batch-level placement information
       map<int, vector<int> > batch_per;
       int objects_per_batch = num_objects / num_batches;
@@ -350,10 +430,12 @@ int CrushTester::test()
 
       // get the total weight of the system
       int total_weight = 0;
-
       for (unsigned i = 0; i < per.size(); i++)
         total_weight += weight[i];
 
+      if (total_weight == 0)
+	continue;
+
       // compute the expected number of objects stored per device in the absence of weighting
       float expected_objects = min(nr, get_maximum_affected_by_rule(r)) * num_objects;
 
@@ -363,6 +445,21 @@ int CrushTester::test()
       for (unsigned i = 0; i < per.size(); i++)
         proportional_weights[i] = (float) weight[i] / (float) total_weight;
 
+      if (output_data_file) {
+        // stage the absolute weight information for post-processing
+        for (unsigned i = 0; i < per.size(); i++) {
+          tester_data.absolute_weights[i] = (float) weight[i] / (float)0x10000;
+        }
+
+        // stage the proportional weight information for post-processing
+        for (unsigned i = 0; i < per.size(); i++) {
+          if (proportional_weights[i] > 0 )
+            tester_data.proportional_weights[i] = proportional_weights[i];
+
+          tester_data.proportional_weights_all[i] = proportional_weights[i];
+        }
+
+      }
       // compute the expected number of objects stored per device when a device's weight is considered
       vector<float> num_objects_expected(num_devices);
 
@@ -384,7 +481,6 @@ int CrushTester::test()
         // create a vector to hold placement results temporarily 
         vector<int> temporary_per ( per.size() );
 
-
         for (int x = batch_min; x <= batch_max; x++) {
           // create a vector to hold the results of a CRUSH placement or RNG simulation
           vector<int> out;
@@ -394,18 +490,17 @@ int CrushTester::test()
               err << "CRUSH"; // prepend CRUSH to placement output
             crush.do_rule(r, x, out, nr, weight);
           } else {
-
-
             if (output_statistics)
               err << "RNG"; // prepend RNG to placement output to denote simulation
-
             // test our new monte carlo placement generator
             random_placement(r, out, nr, weight);
           }
 
-          if (output_statistics){
+          if (output_statistics)
             err << " rule " << r << " x " << x << " " << out << std::endl;
-          }
+
+          if (output_data_file)
+            write_integer_indexed_vector_data_string(tester_data.placement_information, x, out);
 
           for (unsigned i = 0; i < out.size(); i++) {
             per[out[i]]++;
@@ -426,10 +521,10 @@ int CrushTester::test()
       for (unsigned i = 0; i < per.size(); i++)
         if (output_utilization && !output_statistics)
           err << "  device " << i
-          << ":\t" << per[i]
-                          << std::endl;
+          << ":\t" << per[i] << std::endl;
+
       for (map<int,int>::iterator p = sizes.begin(); p != sizes.end(); p++)
-        if ( output_statistics || p->first != nr)
+        if (output_statistics)
           err << "rule " << r << " (" << crush.get_rule_name(r) << ") num_rep " << nr
           << " result size == " << p->first << ":\t"
           << p->second << "/" << (max_x-min_x+1) << std::endl;
@@ -450,6 +545,31 @@ int CrushTester::test()
                 << std::endl;
           }
         }
+
+      if (output_data_file)
+        for (unsigned i = 0; i < per.size(); i++) {
+          vector_data_buffer_f.clear();
+          vector_data_buffer_f.push_back( (float) per[i]);
+          vector_data_buffer_f.push_back( (float) num_objects_expected[i]);
+
+          write_integer_indexed_vector_data_string(tester_data.device_utilization_all, i, vector_data_buffer_f);
+
+          if (num_objects_expected[i] > 0 && per[i] > 0)
+            write_integer_indexed_vector_data_string(tester_data.device_utilization, i, vector_data_buffer_f);
+        }
+
+      if (output_data_file && num_batches > 1) {
+        // stage batch utilization information for post-processing
+        for (int i = 0; i < num_batches; i++) {
+          write_integer_indexed_vector_data_string(tester_data.batch_device_utilization_all, i, batch_per[i]);
+          write_integer_indexed_vector_data_string(tester_data.batch_device_expected_utilization_all, i, batch_per[i]);
+        }
+      }
+
+      string rule_tag = crush.get_rule_name(r);
+
+      if (output_csv)
+        write_data_set_to_csv(output_data_file_name+rule_tag,tester_data);
     }
   }
 
