@@ -32,7 +32,8 @@
 #define CEPH_FEATURE_CRUSH_TUNABLES2 (1<<25)
 #define CEPH_FEATURE_CREATEPOOLID   (1<<26)
 #define CEPH_FEATURE_REPLY_CREATE_INODE   (1<<27)
-#define CEPH_FEATURE_OSD_HBMSGS  (1<<28)
+#define CEPH_FEATURE_OSD_HBMSGS     (1<<28)
+#define CEPH_FEATURE_MDSENC         (1<<29)
 
 /*
  * Features supported.  Should be everything above.
@@ -66,7 +67,8 @@
 	 CEPH_FEATURE_CRUSH_TUNABLES2 |	     \
 	 CEPH_FEATURE_CREATEPOOLID |	     \
 	 CEPH_FEATURE_REPLY_CREATE_INODE |   \
-	 CEPH_FEATURE_OSD_HBMSGS)
+	 CEPH_FEATURE_OSD_HBMSGS | \
+	 CEPH_FEATURE_MDSENC)
 
 #define CEPH_FEATURES_SUPPORTED_DEFAULT  CEPH_FEATURES_ALL
 
