@@ -109,11 +109,11 @@ namespace librbd {
   void image_info(ImageCtx *ictx, image_info_t& info, size_t infosize)
   {
     int obj_order = ictx->order;
-    ictx->md_lock.get_read();
-    ictx->snap_lock.get_read();
+    ictx->md_lock.Lock();
+    ictx->snap_lock.Lock();
     info.size = ictx->get_image_size(ictx->snap_id);
-    ictx->snap_lock.put_read();
-    ictx->md_lock.put_read();
+    ictx->snap_lock.Unlock();
+    ictx->md_lock.Unlock();
     info.obj_size = 1ULL << obj_order;
     info.num_objs = howmany(info.size, ictx->get_object_size());
     info.order = obj_order;
@@ -143,6 +143,7 @@ namespace librbd {
 
   void trim_image(ImageCtx *ictx, uint64_t newsize, ProgressContext& prog_ctx)
   {
+    assert(ictx->md_lock.is_locked());
     CephContext *cct = (CephContext *)ictx->data_ctx.cct();
 
     uint64_t size = ictx->get_current_size();
@@ -246,6 +247,7 @@ namespace librbd {
     uint64_t ver;
 
     if (ictx) {
+      assert(ictx->md_lock.is_locked());
       ictx->refresh_lock.Lock();
       ldout(ictx->cct, 20) << "notify_change refresh_seq = " << ictx->refresh_seq
 			   << " last_refresh = " << ictx->last_refresh << dendl;
@@ -308,6 +310,7 @@ namespace librbd {
   int rollback_image(ImageCtx *ictx, uint64_t snap_id,
 		     ProgressContext& prog_ctx)
   {
+    assert(ictx->md_lock.is_locked());
     uint64_t numseg = ictx->get_num_objects();
     uint64_t bsize = ictx->get_object_size();
 
@@ -357,7 +360,7 @@ namespace librbd {
 	   it != images.end(); ++it) {
 	names.push_back(it->first);
       }
-      if (images.size()) {
+      if (!images.empty()) {
 	last_read = images.rbegin()->first;
       }
       r = images.size();
@@ -428,7 +431,7 @@ namespace librbd {
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->md_lock);
+    Mutex::Locker l(ictx->md_lock);
     do {
       r = add_snap(ictx, snap_name);
     } while (r == -ESTALE);
@@ -472,18 +475,18 @@ namespace librbd {
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->md_lock);
+    Mutex::Locker l(ictx->md_lock);
     snap_t snap_id;
 
     {
       // block for purposes of auto-destruction of l2 on early return
-      RWLock::RLocker l2(ictx->snap_lock);
+      Mutex::Locker l2(ictx->snap_lock);
       snap_id = ictx->get_snap_id(snap_name);
       if (snap_id == CEPH_NOSNAP)
 	return -ENOENT;
 
       parent_spec our_pspec;
-      RWLock::RLocker l3(ictx->parent_lock);
+      Mutex::Locker l3(ictx->parent_lock);
       r = ictx->get_parent_spec(snap_id, &our_pspec);
       if (r < 0) {
 	lderr(ictx->cct) << "snap_remove: can't get parent spec" << dendl;
@@ -526,8 +529,8 @@ namespace librbd {
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->md_lock);
-    RWLock::RLocker l2(ictx->snap_lock);
+    Mutex::Locker l(ictx->md_lock);
+    Mutex::Locker l2(ictx->snap_lock);
     uint64_t features;
     ictx->get_features(ictx->snap_id, &features);
     if ((features & RBD_FEATURE_LAYERING) == 0) {
@@ -546,7 +549,7 @@ namespace librbd {
 
     if (is_protected)
       return -EBUSY;
-
+    
     r = cls_client::set_protection_status(&ictx->md_ctx,
 					  ictx->header_oid,
 					  snap_id,
@@ -569,8 +572,8 @@ namespace librbd {
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->md_lock);
-    RWLock::RLocker l2(ictx->snap_lock);
+    Mutex::Locker l(ictx->md_lock);
+    Mutex::Locker l2(ictx->snap_lock);
     uint64_t features;
     ictx->get_features(ictx->snap_id, &features);
     if ((features & RBD_FEATURE_LAYERING) == 0) {
@@ -658,7 +661,7 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->snap_lock);
+    Mutex::Locker l(ictx->snap_lock);
     bool is_unprotected;
     r = ictx->is_snap_unprotected(snap_name, &is_unprotected);
     // consider both PROTECTED or UNPROTECTING to be 'protected',
@@ -788,15 +791,6 @@ reprotect_and_return_err:
     return r;
   }
 
-  int create(librados::IoCtx& io_ctx, const char *imgname, uint64_t size,
-	     int *order)
-  {
-    CephContext *cct = (CephContext *)io_ctx.cct();
-    bool old_format = cct->_conf->rbd_default_format == 1;
-    uint64_t features = old_format ? 0 : cct->_conf->rbd_default_features;
-    return create(io_ctx, imgname, size, old_format, features, order, 0, 0);
-  }
-
   int create(IoCtx& io_ctx, const char *imgname, uint64_t size,
 	     bool old_format, uint64_t features, int *order,
 	     uint64_t stripe_unit, uint64_t stripe_count)
@@ -829,11 +823,6 @@ reprotect_and_return_err:
     if (!order)
       return -EINVAL;
 
-    if (!*order)
-      *order = cct->_conf->rbd_default_order;
-    if (!*order)
-      *order = RBD_DEFAULT_OBJ_ORDER;
-
     if (*order && (*order > 64 || *order < 12)) {
       lderr(cct) << "order must be in the range [12, 64]" << dendl;
       return -EDOM;
@@ -841,12 +830,8 @@ reprotect_and_return_err:
 
     uint64_t bid = rbd_assign_bid(io_ctx);
 
-    // if striping is enabled, use possibly custom defaults
-    if (!old_format && (features & RBD_FEATURE_STRIPINGV2) &&
-	!stripe_unit && !stripe_count) {
-      stripe_unit = cct->_conf->rbd_default_stripe_unit;
-      stripe_count = cct->_conf->rbd_default_stripe_count;
-    }
+    if (!*order)
+      *order = RBD_DEFAULT_OBJ_ORDER;
 
     // normalize for default striping
     if (stripe_unit == (1ull << *order) && stripe_count == 1) {
@@ -934,13 +919,13 @@ reprotect_and_return_err:
       goto err_close_parent;
     }
 
-    p_imctx->md_lock.get_read();
-    p_imctx->snap_lock.get_read();
+    p_imctx->md_lock.Lock();
+    p_imctx->snap_lock.Lock();
     p_imctx->get_features(p_imctx->snap_id, &p_features);
     size = p_imctx->get_image_size(p_imctx->snap_id);
     p_imctx->is_snap_protected(p_imctx->snap_name, &snap_protected);
-    p_imctx->snap_lock.put_read();
-    p_imctx->md_lock.put_read();
+    p_imctx->snap_lock.Unlock();
+    p_imctx->md_lock.Unlock();
 
     if ((p_features & RBD_FEATURE_LAYERING) != RBD_FEATURE_LAYERING) {
       lderr(cct) << "parent image must support layering" << dendl;
@@ -958,8 +943,7 @@ reprotect_and_return_err:
     if (!order)
       order = p_imctx->order;
 
-    r = create(c_ioctx, c_name, size, false, features, &order,
-	       stripe_unit, stripe_count);
+    r = create(c_ioctx, c_name, size, false, features, &order, stripe_unit, stripe_count);
     if (r < 0) {
       lderr(cct) << "error creating child: " << cpp_strerror(r) << dendl;
       goto err_close_parent;
@@ -984,14 +968,14 @@ reprotect_and_return_err:
       goto err_close_child;
     }
 
-    p_imctx->md_lock.get_write();
+    p_imctx->md_lock.Lock();
     r = ictx_refresh(p_imctx);
-    p_imctx->md_lock.put_write();
+    p_imctx->md_lock.Unlock();
 
     if (r == 0) {
-      p_imctx->snap_lock.get_read();
+      p_imctx->snap_lock.Lock();
       r = p_imctx->is_snap_protected(p_imctx->snap_name, &snap_protected);
-      p_imctx->snap_lock.put_read();
+      p_imctx->snap_lock.Unlock();
     }
     if (r < 0 || !snap_protected) {
       // we lost the race with unprotect
@@ -1072,7 +1056,7 @@ reprotect_and_return_err:
 	return r;
       }
       omap_values.insert(outbl.begin(), outbl.end());
-      if (outbl.size() > 0)
+      if (!outbl.empty())
 	last_read = outbl.rbegin()->first;
     } while (r == MAX_READ);
 
@@ -1090,7 +1074,7 @@ reprotect_and_return_err:
     librados::ObjectWriteOperation op;
     op.create(true);
     op.write_full(databl);
-    if (omap_values.size())
+    if (!omap_values.empty())
       op.omap_set(omap_values);
     r = io_ctx.operate(dst_oid, &op);
     if (r < 0) {
@@ -1161,8 +1145,8 @@ reprotect_and_return_err:
     int r = ictx_check(ictx);
     if (r < 0)
       return r;
-    RWLock::RLocker l(ictx->md_lock);
-    RWLock::RLocker l2(ictx->snap_lock);
+    Mutex::Locker l(ictx->md_lock);
+    Mutex::Locker l2(ictx->snap_lock);
     *size = ictx->get_image_size(ictx->snap_id);
     return 0;
   }
@@ -1172,8 +1156,8 @@ reprotect_and_return_err:
     int r = ictx_check(ictx);
     if (r < 0)
       return r;
-    RWLock::RLocker l(ictx->md_lock);
-    RWLock::RLocker l2(ictx->snap_lock);
+    Mutex::Locker l(ictx->md_lock);
+    Mutex::Locker l2(ictx->snap_lock);
     return ictx->get_features(ictx->snap_id, features);
   }
 
@@ -1182,13 +1166,15 @@ reprotect_and_return_err:
     int r = ictx_check(ictx);
     if (r < 0)
       return r;
-    RWLock::RLocker l(ictx->snap_lock);
-    RWLock::RLocker l2(ictx->parent_lock);
+    Mutex::Locker l(ictx->snap_lock);
+    Mutex::Locker l2(ictx->parent_lock);
     return ictx->get_parent_overlap(ictx->snap_id, overlap);
   }
 
   int open_parent(ImageCtx *ictx)
   {
+    assert(ictx->snap_lock.is_locked());
+    assert(ictx->parent_lock.is_locked());
     string pool_name;
     Rados rados(ictx->md_ctx);
 
@@ -1226,17 +1212,17 @@ reprotect_and_return_err:
       return r;
     }
 
-    ictx->parent->snap_lock.get_write();
+    ictx->parent->snap_lock.Lock();
     r = ictx->parent->get_snap_name(parent_snap_id, &ictx->parent->snap_name);
     if (r < 0) {
       lderr(ictx->cct) << "parent snapshot does not exist" << dendl;
-      ictx->parent->snap_lock.put_write();
+      ictx->parent->snap_lock.Unlock();
       close_image(ictx->parent);
       ictx->parent = NULL;
       return r;
     }
     ictx->parent->snap_set(ictx->parent->snap_name);
-    ictx->parent->snap_lock.put_write();
+    ictx->parent->snap_lock.Unlock();
 
     return 0;
   }
@@ -1248,8 +1234,8 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::RLocker l(ictx->snap_lock);
-    RWLock::RLocker l2(ictx->parent_lock);
+    Mutex::Locker l(ictx->snap_lock);
+    Mutex::Locker l2(ictx->parent_lock);
 
     parent_spec parent_spec;
 
@@ -1278,7 +1264,7 @@ reprotect_and_return_err:
     }
 
     if (parent_snap_name) {
-      RWLock::RLocker l(ictx->parent->snap_lock);
+      Mutex::Locker l(ictx->parent->snap_lock);
       r = ictx->parent->get_snap_name(parent_spec.snap_id,
 				      parent_snap_name);
       if (r < 0) {
@@ -1323,14 +1309,14 @@ reprotect_and_return_err:
       old_format = ictx->old_format;
       unknown_format = false;
       id = ictx->id;
-      ictx->md_lock.get_read();
+      ictx->md_lock.Lock();
       trim_image(ictx, 0, prog_ctx);
-      ictx->md_lock.put_read();
+      ictx->md_lock.Unlock();
 
-      ictx->parent_lock.get_read();
+      ictx->parent_lock.Lock();
       // struct assignment
       parent_info parent_info = ictx->parent_md;
-      ictx->parent_lock.put_read();
+      ictx->parent_lock.Unlock();
 
       r = cls_client::remove_child(&ictx->md_ctx, RBD_CHILDREN,
 				   parent_info.spec, id);
@@ -1387,6 +1373,7 @@ reprotect_and_return_err:
 
   int resize_helper(ImageCtx *ictx, uint64_t size, ProgressContext& prog_ctx)
   {
+    assert(ictx->md_lock.is_locked());
     CephContext *cct = ictx->cct;
 
     if (size == ictx->size) {
@@ -1444,7 +1431,7 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::WLocker l(ictx->md_lock);
+    Mutex::Locker l(ictx->md_lock);
     if (size < ictx->size && ictx->object_cacher) {
       // need to invalidate since we're deleting objects, and
       // ObjectCacher doesn't track non-existent objects
@@ -1467,7 +1454,7 @@ reprotect_and_return_err:
       return r;
     bufferlist bl, bl2;
 
-    RWLock::RLocker l(ictx->snap_lock);
+    Mutex::Locker l(ictx->snap_lock);
     for (map<string, SnapInfo>::iterator it = ictx->snaps_by_name.begin();
 	 it != ictx->snaps_by_name.end(); ++it) {
       snap_info_t info;
@@ -1482,6 +1469,8 @@ reprotect_and_return_err:
 
   int add_snap(ImageCtx *ictx, const char *snap_name)
   {
+    assert(ictx->md_lock.is_locked());
+
     uint64_t snap_id;
 
     int r = ictx->md_ctx.selfmanaged_snap_create(&snap_id);
@@ -1510,12 +1499,14 @@ reprotect_and_return_err:
 
   int rm_snap(ImageCtx *ictx, const char *snap_name)
   {
+    assert(ictx->md_lock.is_locked());
+
     int r;
     if (ictx->old_format) {
       r = cls_client::old_snapshot_remove(&ictx->md_ctx,
 					  ictx->header_oid, snap_name);
     } else {
-      RWLock::RLocker l(ictx->snap_lock);
+      Mutex::Locker l(ictx->snap_lock);
       r = cls_client::snapshot_remove(&ictx->md_ctx,
 				      ictx->header_oid,
 				      ictx->get_snap_id(snap_name));
@@ -1539,7 +1530,7 @@ reprotect_and_return_err:
     ictx->refresh_lock.Unlock();
 
     if (needs_refresh) {
-      RWLock::WLocker l(ictx->md_lock);
+      Mutex::Locker l(ictx->md_lock);
 
       int r = ictx_refresh(ictx);
       if (r < 0) {
@@ -1552,6 +1543,8 @@ reprotect_and_return_err:
   }
 
   int refresh_parent(ImageCtx *ictx) {
+    assert(ictx->snap_lock.is_locked());
+    assert(ictx->parent_lock.is_locked());
     // close the parent if it changed or this image no longer needs
     // to read from it
     int r;
@@ -1565,7 +1558,6 @@ reprotect_and_return_err:
 	  ictx->get_parent_pool_id(ictx->snap_id) ||
 	  ictx->parent->id != ictx->get_parent_image_id(ictx->snap_id) ||
 	  ictx->parent->snap_id != ictx->get_parent_snap_id(ictx->snap_id)) {
-	ictx->clear_nonexistence_cache();
 	close_image(ictx->parent);
 	ictx->parent = NULL;
       }
@@ -1586,6 +1578,7 @@ reprotect_and_return_err:
   int ictx_refresh(ImageCtx *ictx)
   {
     CephContext *cct = ictx->cct;
+    assert(ictx->md_lock.is_locked());
     bufferlist bl, bl2;
 
     ldout(cct, 20) << "ictx_refresh " << ictx << dendl;
@@ -1603,9 +1596,9 @@ reprotect_and_return_err:
     vector<parent_info> snap_parents;
     vector<uint8_t> snap_protection;
     {
-      RWLock::WLocker l(ictx->snap_lock);
+      Mutex::Locker l(ictx->snap_lock);
       {
-	RWLock::WLocker l2(ictx->parent_lock);
+	Mutex::Locker l2(ictx->parent_lock);
 	ictx->lockers.clear();
 	if (ictx->old_format) {
 	  r = read_header(ictx->md_ctx, ictx->header_oid, &ictx->header, NULL);
@@ -1755,12 +1748,12 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::WLocker l(ictx->md_lock);
+    Mutex::Locker l(ictx->md_lock);
     snap_t snap_id;
     uint64_t new_size;
     {
       // need to drop snap_lock before invalidating cache
-      RWLock::RLocker l2(ictx->snap_lock);
+      Mutex::Locker l2(ictx->snap_lock);
       if (!ictx->snap_exists)
 	return -ENOENT;
 
@@ -1831,11 +1824,11 @@ reprotect_and_return_err:
 		   << " -> " << destname << dendl;
     int order = src->order;
 
-    src->md_lock.get_read();
-    src->snap_lock.get_read();
+    src->md_lock.Lock();
+    src->snap_lock.Lock();
     uint64_t src_size = src->get_image_size(src->snap_id);
-    src->snap_lock.put_read();
-    src->md_lock.put_read();
+    src->snap_lock.Unlock();
+    src->md_lock.Unlock();
 
     int r = create(dest_md_ctx, destname, src_size, src->old_format,
 		   src->features, &order, src->stripe_unit, src->stripe_count);
@@ -1861,17 +1854,17 @@ reprotect_and_return_err:
   {
     CopyProgressCtx cp(prog_ctx);
 
-    src->md_lock.get_read();
-    src->snap_lock.get_read();
+    src->md_lock.Lock();
+    src->snap_lock.Lock();
     uint64_t src_size = src->get_image_size(src->snap_id);
-    src->snap_lock.put_read();
-    src->md_lock.put_read();
+    src->snap_lock.Unlock();
+    src->md_lock.Unlock();
 
-    dest->md_lock.get_read();
-    dest->snap_lock.get_read();
+    dest->md_lock.Lock();
+    dest->snap_lock.Lock();
     uint64_t dest_size = dest->get_image_size(dest->snap_id);
-    dest->snap_lock.put_read();
-    dest->md_lock.put_read();
+    dest->snap_lock.Unlock();
+    dest->md_lock.Unlock();
 
     if (dest_size < src_size) {
       lderr(src->cct) << " src size " << src_size << " != dest size " << dest_size << dendl;
@@ -1895,8 +1888,8 @@ reprotect_and_return_err:
 
   int _snap_set(ImageCtx *ictx, const char *snap_name)
   {
-    RWLock::WLocker l1(ictx->snap_lock);
-    RWLock::WLocker l2(ictx->parent_lock);
+    Mutex::Locker l1(ictx->snap_lock);
+    Mutex::Locker l2(ictx->parent_lock);
     int r;
     if ((snap_name != NULL) && (strlen(snap_name) != 0)) {
       r = ictx->snap_set(snap_name);
@@ -1918,12 +1911,6 @@ reprotect_and_return_err:
     // ignore return value, since we may be set to a non-existent
     // snapshot and the user is trying to fix that
     ictx_check(ictx);
-    if (ictx->object_cacher) {
-      // complete pending writes before we're set to a snapshot and
-      // get -EROFS for writes
-      RWLock::WLocker l(ictx->md_lock);
-      ictx->flush_cache();
-    }
     return _snap_set(ictx, snap_name);
   }
 
@@ -1947,9 +1934,9 @@ reprotect_and_return_err:
       }
     }
 
-    ictx->md_lock.get_write();
+    ictx->md_lock.Lock();
     r = ictx_refresh(ictx);
-    ictx->md_lock.put_write();
+    ictx->md_lock.Unlock();
     if (r < 0)
       goto err_close;
 
@@ -1997,36 +1984,28 @@ reprotect_and_return_err:
       return r;
     }
 
-    uint64_t object_size;
-    uint64_t period;
-    uint64_t overlap;
-    uint64_t overlap_periods;
-    uint64_t overlap_objects;
+    Mutex::Locker l(ictx->md_lock);
+    Mutex::Locker l2(ictx->snap_lock);
+    Mutex::Locker l3(ictx->parent_lock);
 
-    {
-      RWLock::RLocker l(ictx->md_lock);
-      RWLock::RLocker l2(ictx->snap_lock);
-      RWLock::RLocker l3(ictx->parent_lock);
-
-      // can't flatten a non-clone
-      if (ictx->parent_md.spec.pool_id == -1) {
-	lderr(ictx->cct) << "image has no parent" << dendl;
-	return -EINVAL;
-      }
-      if (ictx->snap_id != CEPH_NOSNAP || ictx->read_only) {
-	lderr(ictx->cct) << "snapshots cannot be flattened" << dendl;
-	return -EROFS;
-      }
-
-      assert(ictx->parent != NULL);
-      assert(ictx->parent_md.overlap <= ictx->size);
-
-      object_size = ictx->get_object_size();
-      period = ictx->get_stripe_period();
-      overlap = ictx->parent_md.overlap;
-      overlap_periods = (overlap + period - 1) / period;
-      overlap_objects = overlap_periods * ictx->get_stripe_count();
+    // can't flatten a non-clone
+    if (ictx->parent_md.spec.pool_id == -1) {
+      lderr(ictx->cct) << "image has no parent" << dendl;
+      return -EINVAL;
     }
+    if (ictx->snap_id != CEPH_NOSNAP || ictx->read_only) {
+      lderr(ictx->cct) << "snapshots cannot be flattened" << dendl;
+      return -EROFS;
+    }
+
+    assert(ictx->parent != NULL);
+    assert(ictx->parent_md.overlap <= ictx->size);
+
+    uint64_t object_size = ictx->get_object_size();
+    uint64_t period = ictx->get_stripe_period();
+    uint64_t overlap = ictx->parent_md.overlap;
+    uint64_t overlap_periods = (overlap + period - 1) / period;
+    uint64_t overlap_objects = overlap_periods * ictx->get_stripe_count();
 
     char *buf = new char[object_size];
 
@@ -2041,13 +2020,6 @@ reprotect_and_return_err:
       uint64_t object_overlap = ictx->prune_parent_extents(objectx, overlap);
       assert(object_overlap <= object_size);
 
-      RWLock::RLocker l(ictx->parent_lock);
-      // stop early if the parent went away - it just means
-      // another flatten finished first, so this one is useless.
-      if (!ictx->parent) {
-	r = 0;
-	goto err;
-      }
       if ((r = read(ictx->parent, objectx, buf, NULL)) < 0) {
 	lderr(ictx->cct) << "reading from parent failed" << dendl;
 	goto err;
@@ -2076,18 +2048,15 @@ reprotect_and_return_err:
     // and if there are no snaps, remove from the children object as well
     // (if snapshots remain, they have their own parent info, and the child
     // will be removed when the last snap goes away)
-    ictx->snap_lock.get_read();
     if (ictx->snaps.empty()) {
       ldout(ictx->cct, 2) << "removing child from children list..." << dendl;
       int r = cls_client::remove_child(&ictx->md_ctx, RBD_CHILDREN,
 				       ictx->parent_md.spec, ictx->id);
       if (r < 0) {
 	lderr(ictx->cct) << "error removing child from children list" << dendl;
-	ictx->snap_lock.put_read();
 	return r;
       }
     }
-    ictx->snap_lock.put_read();
     notify_change(ictx->md_ctx, ictx->header_oid, NULL, ictx);
 
     ldout(ictx->cct, 20) << "finished flattening" << dendl;
@@ -2108,7 +2077,7 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::RLocker locker(ictx->md_lock);
+    Mutex::Locker locker(ictx->md_lock);
     if (exclusive)
       *exclusive = ictx->exclusive_locked;
     if (tag)
@@ -2145,7 +2114,7 @@ reprotect_and_return_err:
      * checks that we think we will succeed. But for now, let's not
      * duplicate that code.
      */
-    RWLock::RLocker locker(ictx->md_lock);
+    Mutex::Locker locker(ictx->md_lock);
     r = rados::cls::lock::lock(&ictx->md_ctx, ictx->header_oid, RBD_LOCK_NAME,
 			       exclusive ? LOCK_EXCLUSIVE : LOCK_SHARED,
 			       cookie, tag, "", utime_t(), 0);
@@ -2165,7 +2134,7 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    RWLock::RLocker locker(ictx->md_lock);
+    Mutex::Locker locker(ictx->md_lock);
     r = rados::cls::lock::unlock(&ictx->md_ctx, ictx->header_oid,
 				 RBD_LOCK_NAME, cookie);
     if (r < 0)
@@ -2190,7 +2159,7 @@ reprotect_and_return_err:
 		       << "'" << dendl;
       return -EINVAL;
     }
-    RWLock::RLocker locker(ictx->md_lock);
+    Mutex::Locker locker(ictx->md_lock);
     r = rados::cls::lock::break_lock(&ictx->md_ctx, ictx->header_oid,
 				     RBD_LOCK_NAME, cookie, lock_client);
     if (r < 0)
@@ -2461,25 +2430,23 @@ reprotect_and_return_err:
     req->complete(rados_aio_get_return_value(c));
   }
 
-  void rados_ctx_cb(rados_completion_t c, void *arg)
-  {
-    Context *comp = reinterpret_cast<Context *>(arg);
-    comp->complete(rados_aio_get_return_value(c));
-  }
-
   // validate extent against image size; clip to image size if necessary
   int clip_io(ImageCtx *ictx, uint64_t off, uint64_t *len)
   {
-    ictx->md_lock.get_read();
-    ictx->snap_lock.get_read();
+    ictx->md_lock.Lock();
+    ictx->snap_lock.Lock();
     uint64_t image_size = ictx->get_image_size(ictx->snap_id);
     bool snap_exists = ictx->snap_exists;
-    ictx->snap_lock.put_read();
-    ictx->md_lock.put_read();
+    ictx->snap_lock.Unlock();
+    ictx->md_lock.Unlock();
 
     if (!snap_exists)
       return -ENOENT;
 
+    // special-case "len == 0" requests: always valid
+    if (*len == 0)
+      return 0;
+
     // can't start past end
     if (off >= image_size)
       return -EINVAL;
@@ -2491,36 +2458,6 @@ reprotect_and_return_err:
     return 0;
   }
 
-  int aio_flush(ImageCtx *ictx, AioCompletion *c)
-  {
-    CephContext *cct = ictx->cct;
-    ldout(cct, 20) << "aio_flush " << ictx << " completion " << c <<  dendl;
-
-    int r = ictx_check(ictx);
-    if (r < 0)
-      return r;
-
-    ictx->user_flushed();
-
-    c->get();
-    c->add_request();
-    c->init_time(ictx, AIO_TYPE_FLUSH);
-    C_AioWrite *req_comp = new C_AioWrite(cct, c);
-    if (ictx->object_cacher) {
-      ictx->flush_cache_aio(req_comp);
-    } else {
-      librados::AioCompletion *rados_completion =
-	librados::Rados::aio_create_completion(req_comp, NULL, rados_ctx_cb);
-      ictx->data_ctx.aio_flush_async(rados_completion);
-      rados_completion->release();
-    }
-    c->finish_adding_requests(cct);
-    c->put();
-    ictx->perfcounter->inc(l_librbd_aio_flush);
-
-    return 0;
-  }
-
   int flush(ImageCtx *ictx)
   {
     CephContext *cct = ictx->cct;
@@ -2530,7 +2467,6 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    ictx->user_flushed();
     return _flush(ictx);
   }
 
@@ -2570,14 +2506,14 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    ictx->snap_lock.get_read();
+    ictx->snap_lock.Lock();
     snapid_t snap_id = ictx->snap_id;
     ::SnapContext snapc = ictx->snapc;
-    ictx->parent_lock.get_read();
+    ictx->parent_lock.Lock();
     uint64_t overlap = 0;
     ictx->get_parent_overlap(ictx->snap_id, &overlap);
-    ictx->parent_lock.put_read();
-    ictx->snap_lock.put_read();
+    ictx->parent_lock.Unlock();
+    ictx->snap_lock.Unlock();
 
     if (snap_id != CEPH_NOSNAP || ictx->read_only)
       return -EROFS;
@@ -2604,10 +2540,9 @@ reprotect_and_return_err:
 	bl.append(buf + q->first, q->second);
       }
 
-      C_AioWrite *req_comp = new C_AioWrite(cct, c);
       if (ictx->object_cacher) {
-	c->add_request();
-	ictx->write_to_cache(p->oid, bl, p->length, p->offset, req_comp);
+	// may block
+	ictx->write_to_cache(p->oid, bl, p->length, p->offset);
       } else {
 	// reverse map this object extent onto the parent
 	vector<pair<uint64_t,uint64_t> > objectx;
@@ -2616,6 +2551,7 @@ reprotect_and_return_err:
 			      objectx);
 	uint64_t object_overlap = ictx->prune_parent_extents(objectx, overlap);
 
+	C_AioWrite *req_comp = new C_AioWrite(cct, c);
 	AioWrite *req = new AioWrite(ictx, p->oid.name, p->objectno, p->offset,
 				     objectx, object_overlap,
 				     bl, snapc, snap_id, req_comp);
@@ -2655,14 +2591,14 @@ reprotect_and_return_err:
       return r;
 
     // TODO: check for snap
-    ictx->snap_lock.get_read();
+    ictx->snap_lock.Lock();
     snapid_t snap_id = ictx->snap_id;
     ::SnapContext snapc = ictx->snapc;
-    ictx->parent_lock.get_read();
+    ictx->parent_lock.Lock();
     uint64_t overlap = 0;
     ictx->get_parent_overlap(ictx->snap_id, &overlap);
-    ictx->parent_lock.put_read();
-    ictx->snap_lock.put_read();
+    ictx->parent_lock.Unlock();
+    ictx->snap_lock.Unlock();
 
     if (snap_id != CEPH_NOSNAP || ictx->read_only)
       return -EROFS;
@@ -2748,9 +2684,9 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
-    ictx->snap_lock.get_read();
+    ictx->snap_lock.Lock();
     snap_t snap_id = ictx->snap_id;
-    ictx->snap_lock.put_read();
+    ictx->snap_lock.Unlock();
 
     // map
     map<object_t,vector<ObjectExtent> > object_extents;
