@@ -55,9 +55,8 @@ void Dumper::init(int rank)
 
   objecter->set_client_incarnation(0);
 
-  messenger->register_entity(entity_name_t::CLIENT());
   messenger->add_dispatcher_head(this);
-  messenger->start_with_nonce(getpid());
+  messenger->start();
 
   monc->set_want_keys(CEPH_ENTITY_TYPE_MON|CEPH_ENTITY_TYPE_OSD|CEPH_ENTITY_TYPE_MDS);
   monc->set_messenger(messenger);
@@ -67,8 +66,9 @@ void Dumper::init(int rank)
   client_t whoami = monc->get_global_id();
   messenger->set_myname(entity_name_t::CLIENT(whoami.v));
 
+  objecter->init_unlocked();
   lock.Lock();
-  objecter->init();
+  objecter->init_locked();
   objecter->wait_for_osd_map();
   timer.init();
   lock.Unlock();
@@ -78,7 +78,9 @@ void Dumper::shutdown()
 {
   lock.Lock();
   timer.shutdown();
+  objecter->shutdown_locked();
   lock.Unlock();
+  objecter->shutdown_unlocked();
 }
 
 void Dumper::dump(const char *dump_file)
@@ -171,8 +173,10 @@ void Dumper::undump(const char *dump_file)
 
   char buf[200];
   int r = safe_read(fd, buf, sizeof(buf));
-  if (r < 0)
+  if (r < 0) {
+    TEMP_FAILURE_RETRY(::close(fd));
     return;
+  }
 
   long long unsigned start, len;
   sscanf(strstr(buf, "start offset"), "start offset %llu", &start);
@@ -190,7 +194,6 @@ void Dumper::undump(const char *dump_file)
   h.magic = CEPH_FS_ONDISK_MAGIC;
 
   h.layout = g_default_file_layout;
-  h.layout.fl_pg_preferred = -1;
   h.layout.fl_pg_pool = pg_pool;
   
   bufferlist hbl;
@@ -234,6 +237,7 @@ void Dumper::undump(const char *dump_file)
     left -= l;
   }
 
+  TEMP_FAILURE_RETRY(::close(fd));
   cout << "done." << std::endl;
 }
 
