@@ -54,13 +54,10 @@ class MDSMonitor : public PaxosService {
     C_Updated(MDSMonitor *a, MMDSBeacon *c) :
       mm(a), m(c) {}
     void finish(int r) {
-      if (r == -ECANCELED) {
-	if (m)
-	  m->put();
-	return;
-      }
       if (r >= 0)
 	mm->_updated(m);   // success
+      else if (r == -ECANCELED)
+	m->put();
       else
 	mm->dispatch((PaxosServiceMessage*)m);        // try again
     }
@@ -69,7 +66,6 @@ class MDSMonitor : public PaxosService {
 
   void create_new_fs(MDSMap &m, int metadata_pool, int data_pool);
 
-  int parse_mds_id(const char *s, stringstream *pss);
 
   // service methods
   void create_initial();
@@ -97,6 +93,8 @@ class MDSMonitor : public PaxosService {
   void get_health(list<pair<health_status_t,string> >& summary,
 		  list<pair<health_status_t,string> > *detail) const;
   int fail_mds(std::ostream &ss, const std::string &arg);
+  void fail_mds_gid(uint64_t gid);
+
   int cluster_fail(std::ostream &ss);
 
   bool preprocess_command(MMonCommand *m);
