@@ -167,7 +167,6 @@ void PGMonitor::update_from_paxos()
   } 
 
   // walk through incrementals
-  utime_t now(ceph_clock_now(g_ceph_context));
   while (paxosv > pg_map.version) {
     bufferlist bl;
     bool success = paxos->read(pg_map.version+1, bl);
@@ -186,7 +185,7 @@ void PGMonitor::update_from_paxos()
       return;
     }
 
-    pg_map.apply_incremental(inc);
+    pg_map.apply_incremental(g_ceph_context, inc);
     
     dout(10) << pg_map << dendl;
 
@@ -258,6 +257,7 @@ void PGMonitor::encode_pending(bufferlist &bl)
 {
   dout(10) << "encode_pending v " << pending_inc.version << dendl;
   assert(paxos->get_version() + 1 == pending_inc.version);
+  pending_inc.stamp = ceph_clock_now(g_ceph_context);
   pending_inc.encode(bl, mon->get_quorum_features());
 }
 
@@ -1329,6 +1329,15 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
   
   check_full_osd_health(summary, detail, pg_map.full_osds, "full", HEALTH_ERR);
   check_full_osd_health(summary, detail, pg_map.nearfull_osds, "near full", HEALTH_WARN);
+
+  if (pg_map.pg_sum.stats.sum.num_scrub_errors) {
+    ostringstream ss;
+    ss << pg_map.pg_sum.stats.sum.num_scrub_errors << " scrub errors";
+    summary.push_back(make_pair(HEALTH_ERR, ss.str()));
+    if (detail) {
+      detail->push_back(make_pair(HEALTH_ERR, ss.str()));
+    }
+  }
 }
 
 void PGMonitor::check_full_osd_health(list<pair<health_status_t,string> >& summary,
@@ -1336,7 +1345,7 @@ void PGMonitor::check_full_osd_health(list<pair<health_status_t,string> >& summa
 				      const set<int>& s, const char *desc,
 				      health_status_t sev) const
 {
-  if (s.size() > 0) {
+  if (!s.empty()) {
     ostringstream ss;
     ss << s.size() << " " << desc << " osd(s)";
     summary.push_back(make_pair(sev, ss.str()));
