@@ -31,7 +31,6 @@
 #include "os/ObjectStore.h"
 #include "OSDCap.h"
 
-#include "common/DecayCounter.h"
 #include "osd/ClassHandler.h"
 
 #include "include/CompatSet.h"
@@ -133,6 +132,7 @@ class AuthAuthorizeHandlerRegistry;
 
 class OpsFlightSocketHook;
 class HistoricOpsSocketHook;
+class TestOpsSocketHook;
 struct C_CompleteSplits;
 
 extern const coll_t meta_coll;
@@ -149,6 +149,7 @@ public:
     on_deletion_complete.push_front(completion);
   }
   ~DeletingState() {
+    Mutex::Locker l(lock);
     for (list<Context *>::iterator i = on_deletion_complete.begin();
 	 i != on_deletion_complete.end();
 	 ++i) {
@@ -168,6 +169,7 @@ public:
   ObjectStore *&store;
   LogClient &clog;
   PGRecoveryStats &pg_recovery_stats;
+  hobject_t infos_oid;
 private:
   Messenger *&cluster_messenger;
   Messenger *&client_messenger;
@@ -260,7 +262,7 @@ public:
   }
   bool first_scrub_stamp(pair<utime_t, pg_t> *out) {
     Mutex::Locker l(sched_scrub_lock);
-    if (last_scrub_pg.size() == 0)
+    if (last_scrub_pg.empty())
       return false;
     set< pair<utime_t, pg_t> >::iterator iter = last_scrub_pg.begin();
     *out = *iter;
@@ -269,7 +271,7 @@ public:
   bool next_scrub_stamp(pair<utime_t, pg_t> next,
 			pair<utime_t, pg_t> *out) {
     Mutex::Locker l(sched_scrub_lock);
-    if (last_scrub_pg.size() == 0)
+    if (last_scrub_pg.empty())
       return false;
     set< pair<utime_t, pg_t> >::iterator iter = last_scrub_pg.lower_bound(next);
     if (iter == last_scrub_pg.end())
@@ -459,7 +461,7 @@ public:
     return hobject_t(sobject_t(object_t(foo), 0)); 
   }
 
-  hobject_t make_pg_log_oid(pg_t pg) {
+  static hobject_t make_pg_log_oid(pg_t pg) {
     stringstream ss;
     ss << "pglog_" << pg;
     string s;
@@ -467,7 +469,7 @@ public:
     return hobject_t(sobject_t(object_t(s.c_str()), 0));
   }
   
-  hobject_t make_pg_biginfo_oid(pg_t pg) {
+  static hobject_t make_pg_biginfo_oid(pg_t pg) {
     stringstream ss;
     ss << "pginfo_" << pg;
     string s;
@@ -631,6 +633,9 @@ private:
   // -- op tracking --
   OpTracker op_tracker;
   void check_ops_in_flight();
+  void test_ops(std::string command, std::string args, ostream& ss);
+  friend class TestOpsSocketHook;
+  TestOpsSocketHook *test_ops_hook;
   friend class C_CompleteSplits;
 
   // -- op queue --
@@ -1242,10 +1247,8 @@ protected:
       osd->scrub_queue.pop_front();
       return pg;
     }
-    void _process(
-      PG *pg,
-      ThreadPool::TPHandle &handle) {
-      pg->scrub(handle);
+    void _process(PG *pg) {
+      pg->scrub();
       pg->put();
     }
     void _clear() {
@@ -1329,14 +1332,12 @@ protected:
       rep_scrub_queue.pop_front();
       return msg;
     }
-    void _process(
-      MOSDRepScrub *msg,
-      ThreadPool::TPHandle &handle) {
+    void _process(MOSDRepScrub *msg) {
       osd->osd_lock.Lock();
       if (osd->_have_pg(msg->pgid)) {
 	PG *pg = osd->_lookup_lock_pg(msg->pgid);
 	osd->osd_lock.Unlock();
-	pg->replica_scrub(msg, handle);
+	pg->replica_scrub(msg);
 	msg->put();
 	pg->unlock();
       } else {
@@ -1465,7 +1466,7 @@ public:
 public:
   void force_remount();
 
-  int init_op_flags(MOSDOp *op);
+  int init_op_flags(OpRequestRef op);
 
 
   void put_object_context(void *_obc, pg_t pgid);
