@@ -316,6 +316,12 @@ private:
   bool is_in(int osd) const {
     return exists(osd) && !is_out(osd);
   }
+
+  /**
+   * check if an entire crush subtre is down
+   */
+  bool subtree_is_down(int id, set<int> *down_cache) const;
+  bool containing_subtree_is_down(CephContext *cct, int osd, int subtree_type, set<int> *down_cache) const;
   
   int identify_osd(const entity_addr_t& addr) const;
   int identify_osd(const uuid_d& u) const;
@@ -450,6 +456,10 @@ public:
     return -ENOENT;
   }
 
+  int64_t const_lookup_pg_pool_name(const char *name) const {
+    return const_cast<OSDMap *>(this)->lookup_pg_pool_name(name);
+  }
+
   int64_t get_pool_max() const {
     return pool_max;
   }
@@ -543,6 +553,7 @@ public:
   static void build_simple_crush_map_from_conf(CephContext *cct, CrushWrapper& crush,
 					       map<int, const char*>& rulesets);
 
+  bool crush_ruleset_in_use(int ruleset) const;
 
 private:
   void print_osd_line(int cur, ostream *out, Formatter *f) const;
