@@ -883,18 +883,47 @@ protected:
     }
   };
   struct C_OSD_CommittedPushedObject : public Context {
-    ReplicatedPG *pg;
+    boost::intrusive_ptr<ReplicatedPG> pg;
     OpRequestRef op;
-    epoch_t same_since;
+    epoch_t epoch;
     eversion_t last_complete;
-    C_OSD_CommittedPushedObject(ReplicatedPG *p, OpRequestRef o, epoch_t ss, eversion_t lc) : pg(p), op(o), same_since(ss), last_complete(lc) {
-      pg->get();
+    C_OSD_CommittedPushedObject(
+      ReplicatedPG *p, OpRequestRef o, epoch_t epoch, eversion_t lc) :
+      pg(p), op(o), epoch(epoch), last_complete(lc) {
     }
     void finish(int r) {
-      pg->_committed_pushed_object(op, same_since, last_complete);
-      pg->put();
+      pg->_committed_pushed_object(op, epoch, last_complete);
     }
   };
+  struct C_OSD_CompletedPushedObjectReplica : public Context {
+    OSDService *osd;
+    Message *reply;
+    ConnectionRef conn;
+    C_OSD_CompletedPushedObjectReplica (
+      OSDService *osd,
+      Message *reply,
+      ConnectionRef conn) : osd(osd), reply(reply), conn(conn) {}
+    void finish(int) {
+      osd->send_message_osd_cluster(reply, conn.get());
+    }
+  };
+  struct C_OSD_CompletedPull : public Context {
+    boost::intrusive_ptr<ReplicatedPG> pg;
+    hobject_t hoid;
+    epoch_t epoch;
+    C_OSD_CompletedPull(
+      ReplicatedPG *pg,
+      const hobject_t &hoid,
+      epoch_t epoch) : pg(pg), hoid(hoid), epoch(epoch) {}
+    void finish(int) {
+      pg->lock();
+      if (!pg->pg_has_reset_since(epoch)) {
+	pg->finish_recovery_op(hoid);
+      }
+      pg->unlock();
+    }
+  };
+  friend class C_OSD_CompletedPull;
   struct C_OSD_AppliedRecoveredObjectReplica : public Context {
     boost::intrusive_ptr<ReplicatedPG> pg;
     ObjectStore::Transaction *t;
@@ -914,7 +943,7 @@ protected:
   void sub_op_modify_reply(OpRequestRef op);
   void _applied_recovered_object(ObjectStore::Transaction *t, ObjectContext *obc);
   void _applied_recovered_object_replica(ObjectStore::Transaction *t);
-  void _committed_pushed_object(OpRequestRef op, epoch_t same_since, eversion_t lc);
+  void _committed_pushed_object(OpRequestRef op, epoch_t epoch, eversion_t lc);
   void recover_got(hobject_t oid, eversion_t v);
   void sub_op_push(OpRequestRef op);
   void _failed_push(OpRequestRef op);
@@ -929,7 +958,19 @@ protected:
   virtual void _scrub_clear_state();
   virtual void _scrub_finish();
   object_stat_collection_t scrub_cstat;
+  virtual bool _report_snap_collection_errors(
+    const hobject_t &hoid,
+    int osd,
+    const map<string, bufferptr> &attrs,
+    const set<snapid_t> &snapcolls,
+    uint32_t nlinks,
+    ostream &out);
+  virtual void check_snap_collections(
+    ino_t hino, const hobject_t &hoid,
+    const map<string, bufferptr> &attrs,
+    set<snapid_t> *snapcolls);
 
+  virtual void _split_into(pg_t child_pgid, PG *child, unsigned split_bits);
   void apply_and_flush_repops(bool requeue);
 
   void calc_trim_to();
@@ -970,9 +1011,7 @@ private:
   coll_t temp_coll;
   coll_t get_temp_coll(ObjectStore::Transaction *t);
 public:
-  bool have_temp_coll() {
-    return temp_created;
-  }
+  bool have_temp_coll();
   coll_t get_temp_coll() {
     return temp_coll;
   }
