@@ -294,10 +294,26 @@ bool coll_t::is_pg(pg_t& pgid, snapid_t& snap) const
   const char *snap_start = strchr(cstr, '_');
   if (!snap_start)
     return false;
-  if (strncmp(snap_start, "_head", 5) == 0)
+  if (strncmp(snap_start, "_head", 5) == 0) {
     snap = CEPH_NOSNAP;
-  else
+  } else {
+    errno = 0;
     snap = strtoull(snap_start+1, 0, 16);
+    if (errno)
+      return false;
+  }
+  return true;
+}
+
+bool coll_t::is_pg_prefix(pg_t& pgid) const
+{
+  const char *cstr(str.c_str());
+
+  if (!pgid.parse(cstr))
+    return false;
+  const char *snap_start = strchr(cstr, '_');
+  if (!snap_start)
+    return false;
   return true;
 }
 
@@ -509,6 +525,7 @@ void pg_pool_t::dump(Formatter *f) const
   f->dump_unsigned("flags", get_flags());
   f->dump_int("type", get_type());
   f->dump_int("size", get_size());
+  f->dump_int("min_size", get_min_size());
   f->dump_int("crush_ruleset", get_crush_ruleset());
   f->dump_int("object_hash", get_object_hash());
   f->dump_int("pg_num", get_pg_num());
@@ -829,6 +846,7 @@ ostream& operator<<(ostream& out, const pg_pool_t& p)
 {
   out << p.get_type_name()
       << " size " << p.get_size()
+      << " min_size " << p.get_min_size()
       << " crush_ruleset " << p.get_crush_ruleset()
       << " object_hash " << p.get_object_hash_name()
       << " pg_num " << p.get_pg_num()
@@ -1579,15 +1597,9 @@ bool pg_interval_t::check_new_interval(
     i.acting = old_acting;
     i.up = old_up;
 
-    if (i.acting.size() >=
+    if (!i.acting.empty() &&
+	i.acting.size() >=
 	osdmap->get_pools().find(pool_id)->second.min_size) {
-      if (out)
-	*out << "generate_past_intervals " << i
-	     << ": not rw,"
-	     << " up_thru " << lastmap->get_up_thru(i.acting[0])
-	     << " up_from " << lastmap->get_up_from(i.acting[0])
-	     << " last_epoch_clean " << last_epoch_clean
-	     << std::endl;
       if (lastmap->get_up_thru(i.acting[0]) >= i.first &&
 	  lastmap->get_up_from(i.acting[0]) <= i.first) {
 	i.maybe_went_rw = true;
@@ -1699,6 +1711,34 @@ void pg_query_t::generate_test_instances(list<pg_query_t*>& o)
 
 // -- pg_log_entry_t --
 
+string pg_log_entry_t::get_key_name() const
+{
+  char key[40];
+  snprintf(key, sizeof(key), "%010u.%020llu", version.epoch, (long long unsigned)version.version);
+  return string(key);
+}
+
+void pg_log_entry_t::encode_with_checksum(bufferlist& bl) const
+{
+  bufferlist ebl(sizeof(*this)*2);
+  encode(ebl);
+  __u32 crc = ebl.crc32c(0);
+  ::encode(ebl, bl);
+  ::encode(crc, bl);
+}
+
+void pg_log_entry_t::decode_with_checksum(bufferlist::iterator& p)
+{
+  bufferlist bl;
+  ::decode(bl, p);
+  __u32 crc;
+  ::decode(crc, p);
+  if (crc != bl.crc32c(0))
+    throw buffer::malformed_input("bad checksum on pg_log_entry_t");
+  bufferlist::iterator q = bl.begin();
+  decode(q);
+}
+
 void pg_log_entry_t::encode(bufferlist &bl) const
 {
   ENCODE_START(7, 4, bl);
