@@ -9,6 +9,7 @@
 #include "rgw_rest_s3.h"
 #include "rgw_acl.h"
 #include "rgw_policy_s3.h"
+#include "rgw_user.h"
 
 #include "common/armor.h"
 
@@ -66,11 +67,11 @@ static struct response_attr_param resp_attr_params[] = {
   {NULL, NULL},
 };
 
-int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl)
+int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl, off_t bl_ofs, off_t bl_len)
 {
   const char *content_type = NULL;
   string content_type_str;
-  int orig_ret = ret;
+  int req_state = ret;
   map<string, string> response_attrs;
   map<string, string>::iterator riter;
 
@@ -131,9 +132,9 @@ int RGWGetObj_ObjStore_S3::send_response_data(bufferlist& bl)
   }
 
   if (partial_content && !ret)
-    ret = STATUS_PARTIAL_CONTENT;
+    req_state = STATUS_PARTIAL_CONTENT;
 done:
-  set_req_state_err(s, ret);
+  set_req_state_err(s, req_state);
 
   dump_errno(s);
 
@@ -147,8 +148,8 @@ done:
   sent_header = true;
 
 send_data:
-  if (get_data && !orig_ret) {
-    int r = s->cio->write(bl.c_str(), len);
+  if (get_data && !ret) {
+    int r = s->cio->write(bl.c_str() + bl_ofs, bl_len);
     if (r < 0)
       return r;
   }
@@ -276,10 +277,23 @@ void RGWStatBucket_ObjStore_S3::send_response()
   dump_start(s);
 }
 
+static int create_s3_policy(struct req_state *s, RGWRados *store, RGWAccessControlPolicy_S3& s3policy)
+{
+  if (s->has_acl_header) {
+    if (!s->canned_acl.empty())
+      return -ERR_INVALID_REQUEST;
+
+    return s3policy.create_from_headers(store, s->env, s->owner);
+  }
+
+  return s3policy.create_canned(s->owner, s->bucket_owner, s->canned_acl);
+}
+
 int RGWCreateBucket_ObjStore_S3::get_params()
 {
   RGWAccessControlPolicy_S3 s3policy(s->cct);
-  int r = s3policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
+
+  int r = create_s3_policy(s, store, s3policy);
   if (r < 0)
     return r;
 
@@ -315,9 +329,9 @@ int RGWPutObj_ObjStore_S3::get_params()
   if (!s->length)
     return -ERR_LENGTH_REQUIRED;
 
-  int r = s3policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
-  if (!r)
-     return -EINVAL;
+  int r = create_s3_policy(s, store, s3policy);
+  if (r < 0)
+    return r;
 
   policy = s3policy;
 
@@ -898,6 +912,8 @@ int RGWPostObj_ObjStore_S3::get_policy()
     }
 
     s->user = user_info;
+    s->owner.set_id(user_info.user_id);
+    s->owner.set_name(user_info.display_name);
   } else {
     ldout(s->cct, 0) << "No attached policy found!" << dendl;
   }
@@ -907,7 +923,7 @@ int RGWPostObj_ObjStore_S3::get_policy()
 
   RGWAccessControlPolicy_S3 s3policy(s->cct);
   ldout(s->cct, 20) << "canned_acl=" << canned_acl << dendl;
-  if (!s3policy.create_canned(s->user.user_id, "", canned_acl)) {
+  if (s3policy.create_canned(s->owner, s->bucket_owner, canned_acl) < 0) {
     err_msg = "Bad canned ACLs";
     return -EINVAL;
   }
@@ -1114,9 +1130,9 @@ int RGWCopyObj_ObjStore_S3::init_dest_policy()
   RGWAccessControlPolicy_S3 s3policy(s->cct);
 
   /* build a policy for the target object */
-  ret = s3policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
-  if (!ret)
-     return -EINVAL;
+  int r = create_s3_policy(s, store, s3policy);
+  if (r < 0)
+    return r;
 
   dest_policy = s3policy;
 
@@ -1194,12 +1210,19 @@ void RGWGetACLs_ObjStore_S3::send_response()
   s->cio->write(acls.c_str(), acls.size());
 }
 
-int RGWPutACLs_ObjStore_S3::get_canned_policy(ACLOwner& owner, stringstream& ss)
+int RGWPutACLs_ObjStore_S3::get_policy_from_state(RGWRados *store, struct req_state *s, stringstream& ss)
 {
   RGWAccessControlPolicy_S3 s3policy(s->cct);
-  bool r = s3policy.create_canned(owner.get_id(), owner.get_display_name(), s->canned_acl);
-  if (!r)
-    return -EINVAL;
+
+  // bucket-* canned acls do not apply to bucket
+  if (s->object_str.empty()) {
+    if (s->canned_acl.find("bucket") != string::npos)
+      s->canned_acl.clear();
+  }
+
+  int r = create_s3_policy(s, store, s3policy);
+  if (r < 0)
+    return r;
 
   s3policy.to_xml(ss);
 
@@ -1218,9 +1241,9 @@ void RGWPutACLs_ObjStore_S3::send_response()
 int RGWInitMultipart_ObjStore_S3::get_params()
 {
   RGWAccessControlPolicy_S3 s3policy(s->cct);
-  ret = s3policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
-  if (!ret)
-     return -EINVAL;
+  ret = create_s3_policy(s, store, s3policy);
+  if (ret < 0)
+    return ret;
 
   policy = s3policy;
 
@@ -1470,8 +1493,8 @@ RGWOp *RGWHandler_ObjStore_Bucket_S3::op_get()
     return new RGWGetBucketLogging_ObjStore_S3;
   if (is_acl_op()) {
     return new RGWGetACLs_ObjStore_S3;
-  } else if (s->args.exists("uploadId")) {
-    return new RGWListMultipart_ObjStore_S3;
+  } else if (s->args.exists("uploads")) {
+    return new RGWListBucketMultiparts_ObjStore_S3;
   }
   return get_obj_op(true);
 }
@@ -1480,8 +1503,8 @@ RGWOp *RGWHandler_ObjStore_Bucket_S3::op_head()
 {
   if (is_acl_op()) {
     return new RGWGetACLs_ObjStore_S3;
-  } else if (s->args.exists("uploadId")) {
-    return new RGWListMultipart_ObjStore_S3;
+  } else if (s->args.exists("uploads")) {
+    return new RGWListBucketMultiparts_ObjStore_S3;
   }
   return get_obj_op(false);
 }
@@ -1699,6 +1722,8 @@ int RGWHandler_ObjStore_S3::init(RGWRados *store, struct req_state *s, RGWClient
   if (cacl)
     s->canned_acl = cacl;
 
+  s->has_acl_header = s->env->exists_prefix("HTTP_X_AMZ_GRANT");
+
   s->copy_source = s->env->get("HTTP_X_AMZ_COPY_SOURCE");
 
   s->dialect = "s3";
@@ -1868,6 +1893,10 @@ int RGW_Auth_S3::authorize(RGWRados *store, struct req_state *s)
     return -EPERM;
   }
 
+  // populate the owner info
+  s->owner.set_id(s->user.user_id);
+  s->owner.set_name(s->user.display_name);
+
   /* now verify signature */
    
   string auth_hdr;
